/* LOAD MODULE DEPENDENCIES */

var debug               = require("debug")("api:json_object_class");
var config              = require('../../config');
var uuid              = require('uuid/v1');
var moment              = require('moment');
var request             = require('request');

/* PROPERTIES */

exports.INFO = "INFO",
exports.PAYLOAD_DATA = "PAYLOAD_DATA";
exports.JOURNEY = "JOURNEY";
exports.REMARK = "REMARK";

/* METHODS */

// extract service request type from json object
exports.getServiceRequestType = function getServiceRequestType(jsonData, callback) {
    debug("Getting get service request type");

    var serviceRequestType = null;

    try {
        // get service request type
        serviceRequestType = jsonData.INFO.serviceRequestType === null ? "" : jsonData.INFO.serviceRequestType;
    } catch (ex) {
        // error callback
        if(!serviceRequestType) callback(new Error("Unknown Service Type Request"));
    }
    // return service request type to callback
    callback(null, serviceRequestType);
};

// set service request type from json object
exports.setServiceRequestType = function setServiceRequestType(jsonData, serviceRequestType, callback) {
    debug("Setting get service request type");

    try {
        // set service request type
        jsonData.INFO.serviceRequestType = serviceRequestType;
    } catch (ex) {
        // error callback
        jsonData.INFO.serviceRequestType = "";
    }
    // return service request type attached Json Object back to callback
    callback(null, jsonData);
};

// extract data set to a receiver system
exports.getSentData = function getSentData(jsonData, callback) {
    debug("Getting sent data");

    var data = null;

    try {
        // get data from the requesting service in the last journey
        data = jsonData.JOURNEY[Number(jsonData.INFO.lastSystemIndex)].data;
    } catch (ex) {
        // error callback
        if(!data) callback(new Error("No Data Found In THE JSON Object!"));
    }
    // return data to callback
    callback(null, data);
};

// append a journey trace in the jsonObject
exports.appendJourney = function appendJourney(jsonData, dataToBeSent, callback) {
    debug("Append Journey data to Json Object");

    // if data to be sent is not a valid json then stringfy the data
    //if(!exports.isValidJson(dataToBeSent)) y;

    // prepare data to be appended
    var appendedData = {
        "journeyOrderId": Number(jsonData.INFO.lastSystemIndex) + 1,
        "systemId": config.SYSTEM_ID,
        "systemName": config.SYSTEM_NAME,
        "createdTime": moment().format("YYYY-MM-DD HH:mm:ss SSS"),
        "data": dataToBeSent
    };

    // append it to jsonData
    jsonData.JOURNEY.push(appendedData);

    // modify last service index
    jsonData.INFO.lastSystemIndex = jsonData.JOURNEY.length - 1;

    // return updated jsonData
    callback(null, jsonData);
};

// get data sent from other service and append a journey trace in the jsonObject
exports.getSentDataAndAppendJourney = function getSentDataAndAppendJourney(jsonData, callback) {
    debug("Get data sent and Append Journey data to Json Object");
    var errMessage = {
        name: 'INVALID_JSON_OBJECT_RECEIVED',
        message: 'Invalid Json Object Sent from requesting server!',
        status: 500,
        jsonData: jsonData
    };
    if(!jsonData) return callback(errMessage);

    // stringfy the json object
    try {
        jsonData = JSON.parse(jsonData);
    } catch (e) {
        return callback(errMessage);
    }    

    var dataSent = null;
    
    // check validity of json
    exports.isValidJsonObject(jsonData, function(err, result){
        if(err) return callback(true);
        // get data from the requesting service in the last journey
        dataSent = jsonData.JOURNEY[Number(jsonData.INFO.lastSystemIndex)].data;
    
        // error callback
        if(!dataSent) dataSent = {}; //callback(new Error("No Data Found In THE JSON Object!"));

        // prepare data to be appended
        var appendedData = {
            "journeyOrderId": Number(jsonData.INFO.lastSystemIndex) + 1,
            "systemId": config.SYSTEM_ID,
            "systemName": config.SYSTEM_NAME,
            "createdTime": moment().format("YYYY-MM-DD HH:mm:ss.SSS"),
            "data": dataSent
        };

        //console.log(jsonData);

        if(!jsonData.JOURNEY) jsonData.JOURNEY = [];

        // append it to jsonData
        jsonData.JOURNEY.push(appendedData);

        if(!jsonData.INFO) jsonData.INFO = {};

        // modify last service index
        jsonData.INFO.lastSystemIndex = jsonData.JOURNEY.length - 1;
        console.log(jsonData);
        // return updated jsonData
        callback(null, dataSent, jsonData);
    });
    
    
};

// get payload data
exports.getPayloadData = function getPayloadData (jsonData, callback) {
    debug("Getting payload data from json Object");

    var payloadData = null;

    try{
        // extract the payload data from jsonData
        payloadData = jsonData[this.PAYLOAD_DATA];

    } catch(ex) {
        // if there is an error
        if(!payloadData) callback(new Error("Payload data not appended: " + ex));
    }

    // return payload data to the callback
    callback(null, payloadData);
};

// create json object
exports.createJsonObject = function createJsonObject(payloadData, dataToBeSent, serviceRequestType,  callback ) {
    debug ("Creating a json object...");

    // Construct the default json object for gateway manager
    var data = {
        "INFO" : {
            "id": uuid(),
            "createdTime": moment().format("YYYY-MM-DD HH:mm:ss.SSS"),
            "lastSystemIndex": 0,
            "serviceRequestType": serviceRequestType || ""
        },
        "PAYLOAD_DATA": payloadData,
        "JOURNEY": [
            {
                "journeyOrderId": 0,
                "systemId": config.SYSTEM_ID,
                "systemName": config.SYSTEM_NAME,
                "createdTime": moment().format("YYYY-MM-DD HH:mm:ss.SSS"),
                "data": dataToBeSent
            }
        ],
        "REMARK": {},
        "STATUS": {
            "error":false,
            "status":"200",
            "message":"valid hisab json object",
            "data": {}
        }
    };

    // return the data
    callback(null, data);
};

exports.isValidJsonObject = function isValidJsonObject(jsonObject, callback) {
    debug("Check if JSON object valid");
    var errMessage = {
        name: 'INVALID_JSON_OBJECT_RECEIVED',
        message: 'Invalid Json Object Sent from requesting server!',
        decription: 'JsonObjectReceived: ' + JSON.stringify(jsonObject),
        status: 500
    };
    // check if the INFO field exist
    if(jsonObject.INFO === null  || typeof jsonObject.INFO !== "object") {
        callback(errMessage);
        return;
    }
    // if INFO.id exist
    if(jsonObject.INFO.id === null || typeof jsonObject.INFO.id !== "string") {
        debug(" -- error in INFO.id");
        callback(errMessage);
        return;
    }
    // if INFO.createdTime exist
    // if(jsonObject.INFO.createdTime === null || typeof jsonObject.INFO.createdTime !== "string") {
    //     debug(" -- error in INFO.createdTime");
    //     callback(true);
    //     return;
    // }
    // if INFO.lastSystemIndex exist
    if(jsonObject.INFO.lastSystemIndex === null || !(typeof jsonObject.INFO.lastSystemIndex === "number")) {
        debug(" -- error in INFO.lastSystemIndex");
        callback(errMessage);
        return;
    }

    // if INFO.serviceRequestType exist
    // if(jsonObject.INFO.serviceRequestType === null || typeof jsonObject.INFO.serviceRequestType !== "string") {
    //     debug(" -- error in INFO.serviceRequestType");
    //     callback(true);
    //     return;
    // }

    // if PAYLOAD_DATA exist
    // if(jsonObject.PAYLOAD_DATA === null || typeof jsonObject.PAYLOAD_DATA !== "object") {
    //     debug(" -- error in PAYLOAD_DATA");
    //     callback(true);
    //     return;
    // }

    // if JOURNEY exist
    // if(jsonObject.JOURNEY === null || typeof jsonObject.JOURNEY !== "object") {
    //     debug(" -- error in JOURNEY");
    //     callback(true);
    //     return;
    // }

    // if REMARK exist
    // if(jsonObject.REMARK === null || typeof jsonObject.REMARK !== "object") {
    //     debug(" -- error in REMARK");
    //     callback(true);
    //     return;
    // }

    return callback(null, true);
};

var isValidJsonObject = function isValidJsonObject(jsonObject, cb) {
    exports.isValidJsonObject(jsonObject, function (err, isValid){
        if(err) return cb(true);
        debug("-- valid JsonObject Requested!");
        // if no error, the json object is vaid!
        return cb(null, true);
    });
};

exports.validateJsonObject = function ValidJsonObject(options) {
    options = options || {};
    debug(" - JsonObject validator middleware started!");

    return function middleware(req, res, next) {
        var body = req.body;
        var errMessage = {
            name: 'INVALID_JSON_OBJECT_RECEIVED',
            message: 'Invalid Json Object Sent from requesting server!',
            decription: 'JsonObjectReceived: ' + JSON.stringify(body),
            status: 500
        };
        // if there is a body
        if(body) {

            isValidJsonObject(body, function cb(err, isValid){
                // if the body is a valid body
                if(isValid) {
                    debug("-- valid jsonObject");
                    // continue
                    next();
                } else {
                    debug("-- Invalid jsonObject!");
                    res.status(400).json(errMessage);
                    return;
                }
            });


        } else {
            debug("-- Invalid jsonObject");
            res.status(400).json(errMessage);
            return;
        }
    };
};

exports.isJsonIsError = function (jsonObject, cb) {
    try {
        jsonObject = JSON.parse(jsonObject);
    } catch (e) {
        return cb(e);
    } 
    console.log("[JSON-OBJECT TO BE CHECKED]", jsonObject);
    if(!jsonObject) {
        return cb({
            error: true,
            message: "Incorrect HISAB JSON Object"
        }, jsonObject); 
    }
    var STATUS = jsonObject["STATUS"];
    jsonObject = JSON.stringify(jsonObject);
    console.log(STATUS);
    if (STATUS === undefined || typeof STATUS["error"] === undefined){
        return cb({
            error: true,
            message: "Incorrect HISAB JSON Object"
        }, jsonObject); 
    } 
    return STATUS["error"]? cb(STATUS, jsonObject): cb(null, jsonObject);
}
/**
 * UTILITY FUNCTIONS
 */

exports.requestPOST  = function requestPost(URL, data, callback) {
    var headers = {
        'User-Agent': config.SYSTEM_NAME,
        'Content-Type': 'application/json'
    };

    // stringfy the json
    data = JSON.stringify(data);

    // Configure the request
    var options = {
        url: URL,
        method: 'POST',
        headers: headers,
        body: data
    };

    // request user manager API
    request(options, function (err, response, body) {
        return callback(err, response, body);
    });
};

exports.requestGET  = function requestGET(URL, callback) {
    var headers = {
        'User-Agent': config.SYSTEM_NAME,
        'Content-Type': 'application/json'
    };

    // Configure the request
    var options = {
        url: URL,
        method: 'GET',
        headers: headers
    };

    // request user manager API
    request(options, function (err, response, body) {
        return callback(err, response, body);
    });
};

exports.isValidJson = function isValidJson(str, callback) {
    try {
        JSON.parse(str);
        return true;
    } catch (e) {
        return false;
    }
};